'''This module implements the client side of the FinalConfig protocol.

Important: This module is meant for applications, that use dbus in an asynchronous
fashion based on the glib mainloop. Such an application typically contains the following
line of code:
dbus.mainloop.glib.DBusGMainLoop (set_as_default=True)
'''

import os.path
import xml.dom.minidom
import weakref

import gtk
import dbus

import finalconfig.config
import finalconfig.client.shared


class Config:
    '''This class implements handling of the user preferences data and the interaction
    with FinalConfig.'''
    
    def __init__ (self, desc_path, config_path, app_name):
        '''Set up FinalConfig support.
        
        desc_path: Name of the default description file.
        
        config_path: Name of the default config file (the config file is read in at this
        point). If the file does not exist it is created (with default settings).
        If the file is outdatd, new default settings will be added).
        
        app_name: Name of the application as used in the preference dialogue window title.
        '''
        self.desc_path = os.path.abspath (desc_path)
        self.config_path = os.path.abspath (config_path)
        self.app_name = app_name
        self.proxy = None
        self.listener = []
        finalconfig.client.shared.ensure_config (self.config_path, self.desc_path)
        self._read_config (self.config_path)

    def edit (self):
        '''Request editing the user preferences.'''
        if self.proxy is None:
            bus = dbus.SessionBus()
            proxy = bus.get_object ('de.zpages.FinalConfig', '/')
            handle = proxy.start_session (self.config_path, self.desc_path, self.app_name)
            self.proxy = bus.get_object ('de.zpages.FinalConfig', handle)
            self.proxy.connect_to_signal ('reply', self._reply, 'de.zpages.FinalConfig')
            self.proxy.connect_to_signal ('close_session', self._close,
                 'de.zpages.FinalConfig')        
        else:
            self.proxy.highlight()
        
    def shutdown (self):
        '''Request to shut down all ongoing preferences dialogues for this application.'''
        if self.proxy is not None:
            self.proxy.abort_session()

    def add_listener (self, listener):
        '''Add a listener to be called, when the user has changed the preference settings.
        
        The listener must implement a function:
        on_config_changed (self, config)
        where config is the finalconfig.client.async.config object.
        Note: That the listener is stored as a weak reference.
        '''
        self.listener.append (weakref.ref (listener))
        self.listener = filter (lambda l: l() is not None, self.listener)

    def remove_listener (self, listener):
        '''Remove listener, that was previously added with add_listener.'''
        self.listener.remove (listener)

    def _reply (self, config_path):
        self._read_config (config_path)
        for listener in self.listener:
            if listener() is not None:
                listener().on_config_changed (self)

    def _close (self):
        self.proxy = None
        
    def _read_config (self, config_path):
        self.config = finalconfig.config.read_config (self.config_path)

